; ******************************************************
; BASIC .ASM template file for AVR
; ******************************************************

.include "C:\VMLAB\include\tn15def.inc"
; Define here the variables
;
.def  temp  =r16
.def	bitzaehler	=r20
.def	datum		=r17
.def	null		= r18
.def	rT0run 	= r19
.def	adresse	= r22
.def	kommando	= r23
.def	zeit		= r21
.def	helligkeit = r24
.def	anausz	= r25
.def	stufe		= r1

.equ	T0run	= 3 ; Zeitgeber 0 mit Takt/64 laufen lassen
.equ	fosc	= 1600000 ; Taktfrequenz der CPU
.equ	stufenw = 5
.equ	schwelle = 38  ; 1.5e-3/TeilerT0*fosc
.equ	cal	= 5e	; Kalibrierungsbyte, mit "make read_cal" auslesen!

.equ	adresse = 18 ; Gerteadresse
.equ	heller = 20 ; Kommando fr "heller"
.equ 	dunkler = 21; Kommando fr "dunkler"
.equ	einaus = 22 ; Kommando fr "ein/aus"

; Define here Reset and interrupt vectors, if any
;
reset:
   rjmp start
.org INT0addr
	rjmp	neuesBit
.org T0OVaddr
	rjmp	zeitUeber
	
.org	0x13
; Program starts here after Reset
;
start:
	ldi	temp,cal
	out	osccal,temp	; kalibrieren
	clr	bitzaehler
	clr	null
	clr	datum
	clr	anausz
	ldi	rT0run,T0run
	ldi	temp,stufenw
	mov	stufe,temp
	sbi	DDRB,1	; OC1A als Ausgang	
	ldi	temp,255
	out	OCR1B,temp
	ldi	temp,((1<<PWM1)+(1<<COM1A1)+(1<<COM1A0)+1) ; schnelle PWM, invertiert, voller CPU-Takt
	out	TCCR1,temp
	clr	helligkeit
	out 	OCR1A,helligkeit
	ldi	temp,(1<<SE)+(1<<ISC01); enable sleep, sleep mode = idle, INT0 auf fallender Flanke
	out	MCUCR,temp
	ldi	temp,(1<<INT0) ; schalte INT0 frei
	out	GIMSK,temp
	ldi	temp,(1<<TOIE0) ; T0 berlaufinterrupt
	out	TIMSK,temp		
   sei	; erlaube Unterbrechungen
forever:
	sleep
rjmp forever

neuesBit:
	in		zeit,TCNT0
	; Signal stabil?
	ldi	temp,5
stabil:
	sbic	PORTB,2	; solange INT0 unten
	reti
	dec	temp
	brne	stabil
	out	TCNT0,null	
	inc 	bitzaehler
	; was nun?
	cpi	bitzaehler,1
	brne	prfLetztesBit
	clr	datum
	out	TCCR0,rT0run
	reti
prfLetztesBit:
	cpi	bitzaehler,16
	brne	prfEndeBits
	out	TCCR0,null	; T0 stoppen
	clr	bitzaehler
	mov	kommando,datum
	rcall auswertung
	reti
prfEndeBits:			
	cpi	bitzaehler,15
	brne	prfAdresse
	reti
prfAdresse:
	cpi	bitzaehler,7
	brne	normalesBit
	mov	adresse,datum
	clr	datum		
normalesBit:
	lsr	datum	; Platz schaffen
	cpi	zeit,schwelle
	brlo	bitFertig
	ori	datum,128
bitFertig:	
	reti
	
	; T0 bergelaufen, System zurcksetzen
zeitUeber:
	clr bitzaehler
	clr datum
	out TCCR0,null
	reti
	
auswertung:
	; Achtung, Adresse mit 8 multipliziert!
	cpi	adresse,(8*adresse)
	brne	auswertung_ende
	
	cpi	kommando,heller
	brne	auswertung_dunkler
	cpi	helligkeit,255
	breq	auswertung_ende
	add	helligkeit,stufe
	rjmp	auswertung_ende
auswertung_dunkler:
	cpi	kommando,dunkler
	brne	auswertung_anaus
	cpi	helligkeit,0
	breq	auswertung_ende	
	sub	helligkeit,stufe
	rjmp	auswertung_ende
auswertung_anaus:
	cpi	kommando,einaus
	brne	auswertung_ende
	inc	anausz
	cpi	anausz,5
	brne	auswertung_ende
	clr	anausz
	mov	temp,helligkeit
	ldi	helligkeit,255
	cpse	temp,null
	clr 	helligkeit				
auswertung_ende:
	out 	OCR1A,helligkeit
	ret	
	



